@@ -36,6 +36,7 @@ require_once dirname( __DIR__ ) . '/wp-load.php';
 nocache_headers();
 
 if ( get_option( 'db_upgraded' ) ) {
+
 	flush_rewrite_rules();
 	update_option( 'db_upgraded', false );
 
@@ -45,7 +46,11 @@ if ( get_option( 'db_upgraded' ) ) {
 	 * @since 2.8.0
 	 */
 	do_action( 'after_db_upgrade' );
-} elseif ( get_option( 'db_version' ) != $wp_db_version && empty( $_POST ) ) {
+
+} elseif ( ! wp_doing_ajax() && empty( $_POST )
+	&& (int) get_option( 'db_version' ) !== $wp_db_version
+) {
+
 	if ( ! is_multisite() ) {
 		wp_redirect( admin_url( 'upgrade.php?_wp_http_referer=' . urlencode( wp_unslash( $_SERVER['REQUEST_URI'] ) ) ) );
 		exit;
@@ -111,16 +116,16 @@ $time_format = __( 'g:i a' );
 wp_enqueue_script( 'common' );
 
 /**
- * $pagenow is set in vars.php
- * $wp_importers is sometimes set in wp-admin/includes/import.php
- * The remaining variables are imported as globals elsewhere, declared as globals here
+ * $pagenow is set in vars.php.
+ * $wp_importers is sometimes set in wp-admin/includes/import.php.
+ * The remaining variables are imported as globals elsewhere, declared as globals here.
  *
- * @global string $pagenow
+ * @global string $pagenow      The filename of the current screen.
  * @global array  $wp_importers
  * @global string $hook_suffix
  * @global string $plugin_page
- * @global string $typenow
- * @global string $taxnow
+ * @global string $typenow      The post type of the current screen.
+ * @global string $taxnow       The taxonomy of the current screen.
  */
 global $pagenow, $wp_importers, $hook_suffix, $plugin_page, $typenow, $taxnow;
 
@@ -257,7 +262,9 @@ if ( isset( $plugin_page ) ) {
 			wp_die( __( 'Invalid plugin page.' ) );
 		}
 
-		if ( ! ( file_exists( WP_PLUGIN_DIR . "/$plugin_page" ) && is_file( WP_PLUGIN_DIR . "/$plugin_page" ) ) && ! ( file_exists( WPMU_PLUGIN_DIR . "/$plugin_page" ) && is_file( WPMU_PLUGIN_DIR . "/$plugin_page" ) ) ) {
+		if ( ! ( file_exists( WP_PLUGIN_DIR . "/$plugin_page" ) && is_file( WP_PLUGIN_DIR . "/$plugin_page" ) )
+			&& ! ( file_exists( WPMU_PLUGIN_DIR . "/$plugin_page" ) && is_file( WPMU_PLUGIN_DIR . "/$plugin_page" ) )
+		) {
 			/* translators: %s: Admin page generated by a plugin. */
 			wp_die( sprintf( __( 'Cannot load %s.' ), htmlentities( $plugin_page ) ) );
 		}
@@ -289,7 +296,7 @@ if ( isset( $plugin_page ) ) {
 
 	require_once ABSPATH . 'wp-admin/admin-footer.php';
 
-	exit();
+	exit;
 } elseif ( isset( $_GET['import'] ) ) {
 
 	$importer = $_GET['import'];
@@ -313,13 +320,24 @@ if ( isset( $plugin_page ) ) {
 	 *
 	 * The dynamic portion of the hook name, `$importer`, refers to the importer slug.
 	 *
+	 * Possible hook names include:
+	 *
+	 *  - `load-importer-blogger`
+	 *  - `load-importer-wpcat2tag`
+	 *  - `load-importer-livejournal`
+	 *  - `load-importer-mt`
+	 *  - `load-importer-rss`
+	 *  - `load-importer-tumblr`
+	 *  - `load-importer-wordpress`
+	 *
 	 * @since 3.5.0
 	 */
 	do_action( "load-importer-{$importer}" ); // phpcs:ignore WordPress.NamingConventions.ValidHookName.UseUnderscores
 
+	// Used in the HTML title tag.
+	$title        = __( 'Import' );
 	$parent_file  = 'tools.php';
 	$submenu_file = 'import.php';
-	$title        = __( 'Import' );
 
 	if ( ! isset( $_GET['noheader'] ) ) {
 		require_once ABSPATH . 'wp-admin/admin-header.php';
@@ -330,7 +348,7 @@ if ( isset( $plugin_page ) ) {
 	define( 'WP_IMPORTING', true );
 
 	/**
-	 * Whether to filter imported data through kses on import.
+	 * Filters whether to filter imported data through kses on import.
 	 *
 	 * Multisite uses this hook to filter all data through kses by default,
 	 * as a super administrator may be assisting an untrusted user.
@@ -350,7 +368,7 @@ if ( isset( $plugin_page ) ) {
 	// Make sure rules are flushed.
 	flush_rewrite_rules( false );
 
-	exit();
+	exit;
 } else {
 	/**
 	 * Fires before a particular screen is loaded.
@@ -358,7 +376,7 @@ if ( isset( $plugin_page ) ) {
 	 * The load-* hook fires in a number of contexts. This hook is for core screens.
 	 *
 	 * The dynamic portion of the hook name, `$pagenow`, is a global variable
-	 * referring to the filename of the current page, such as 'admin.php',
+	 * referring to the filename of the current screen, such as 'admin.php',
 	 * 'post-new.php' etc. A complete hook for the latter would be
 	 * 'load-post-new.php'.
 	 *
