@@ -150,6 +150,15 @@ function determine_locale() {
 		( isset( $_GET['_locale'] ) && 'user' === $_GET['_locale'] && wp_is_json_request() )
 	) {
 		$determined_locale = get_user_locale();
+	} elseif (
+		( ! empty( $_REQUEST['language'] ) || isset( $GLOBALS['wp_local_package'] ) )
+		&& wp_installing()
+	) {
+		if ( ! empty( $_REQUEST['language'] ) ) {
+			$determined_locale = sanitize_locale_name( $_REQUEST['language'] );
+		} else {
+			$determined_locale = $GLOBALS['wp_local_package'];
+		}
 	}
 
 	if ( ! $determined_locale ) {
@@ -161,7 +170,7 @@ function determine_locale() {
 	 *
 	 * @since 5.0.0
 	 *
-	 * @param string $locale The locale.
+	 * @param string $determined_locale The locale.
 	 */
 	return apply_filters( 'determine_locale', $determined_locale );
 }
@@ -719,6 +728,10 @@ function load_textdomain( $domain, $mofile, $locale = null ) {
 
 	$l10n_unloaded = (array) $l10n_unloaded;
 
+	if ( ! is_string( $domain ) ) {
+		return false;
+	}
+
 	/**
 	 * Filters whether to short-circuit loading .mo file.
 	 *
@@ -780,32 +793,73 @@ function load_textdomain( $domain, $mofile, $locale = null ) {
 	 */
 	$mofile = apply_filters( 'load_textdomain_mofile', $mofile, $domain );
 
-	if ( ! is_readable( $mofile ) ) {
-		return false;
-	}
-
 	if ( ! $locale ) {
 		$locale = determine_locale();
 	}
 
-	$mo = new MO();
-	if ( ! $mo->import_from_file( $mofile ) ) {
-		$wp_textdomain_registry->set( $domain, $locale, false );
+	$i18n_controller = WP_Translation_Controller::get_instance();
 
-		return false;
+	// Ensures the correct locale is set as the current one, in case it was filtered.
+	$i18n_controller->set_locale( $locale );
+
+	/**
+	 * Filters the preferred file format for translation files.
+	 *
+	 * Can be used to disable the use of PHP files for translations.
+	 *
+	 * @since 6.5.0
+	 *
+	 * @param string $preferred_format Preferred file format. Possible values: 'php', 'mo'. Default: 'php'.
+	 * @param string $domain           The text domain.
+	 */
+	$preferred_format = apply_filters( 'translation_file_format', 'php', $domain );
+	if ( ! in_array( $preferred_format, array( 'php', 'mo' ), true ) ) {
+		$preferred_format = 'php';
 	}
 
-	if ( isset( $l10n[ $domain ] ) ) {
-		$mo->merge_with( $l10n[ $domain ] );
+	$translation_files = array();
+
+	if ( 'mo' !== $preferred_format ) {
+		$translation_files[] = substr_replace( $mofile, ".l10n.$preferred_format", - strlen( '.mo' ) );
 	}
 
-	unset( $l10n_unloaded[ $domain ] );
+	$translation_files[] = $mofile;
 
-	$l10n[ $domain ] = &$mo;
+	foreach ( $translation_files as $file ) {
+		/**
+		 * Filters the file path for loading translations for the given text domain.
+		 *
+		 * Similar to the {@see 'load_textdomain_mofile'} filter with the difference that
+		 * the file path could be for an MO or PHP file.
+		 *
+		 * @since 6.5.0
+		 * @since 6.6.0 Added the `$locale` parameter.
+		 *
+		 * @param string $file   Path to the translation file to load.
+		 * @param string $domain The text domain.
+		 * @param string $locale The locale.
+		 */
+		$file = (string) apply_filters( 'load_translation_file', $file, $domain, $locale );
 
-	$wp_textdomain_registry->set( $domain, $locale, dirname( $mofile ) );
+		$success = $i18n_controller->load_file( $file, $domain, $locale );
 
-	return true;
+		if ( $success ) {
+			if ( isset( $l10n[ $domain ] ) && $l10n[ $domain ] instanceof MO ) {
+				$i18n_controller->load_file( $l10n[ $domain ]->get_filename(), $domain, $locale );
+			}
+
+			// Unset NOOP_Translations reference in get_translations_for_domain().
+			unset( $l10n[ $domain ] );
+
+			$l10n[ $domain ] = new WP_Translations( $i18n_controller, $domain );
+
+			$wp_textdomain_registry->set( $domain, $locale, dirname( $file ) );
+
+			return true;
+		}
+	}
+
+	return false;
 }
 
 /**
@@ -857,6 +911,11 @@ function unload_textdomain( $domain, $reloadable = false ) {
 	 */
 	do_action( 'unload_textdomain', $domain, $reloadable );
 
+	// Since multiple locales are supported, reloadable text domains don't actually need to be unloaded.
+	if ( ! $reloadable ) {
+		WP_Translation_Controller::get_instance()->unload_textdomain( $domain );
+	}
+
 	if ( isset( $l10n[ $domain ] ) ) {
 		if ( $l10n[ $domain ] instanceof NOOP_Translations ) {
 			unset( $l10n[ $domain ] );
@@ -895,7 +954,7 @@ function load_default_textdomain( $locale = null ) {
 	}
 
 	// Unload previously loaded strings so we can switch translations.
-	unload_textdomain( 'default' );
+	unload_textdomain( 'default', true );
 
 	$return = load_textdomain( 'default', WP_LANG_DIR . "/$locale.mo", $locale );
 
@@ -936,6 +995,10 @@ function load_plugin_textdomain( $domain, $deprecated = false, $plugin_rel_path
 	/** @var WP_Textdomain_Registry $wp_textdomain_registry */
 	global $wp_textdomain_registry;
 
+	if ( ! is_string( $domain ) ) {
+		return false;
+	}
+
 	/**
 	 * Filters a plugin's locale.
 	 *
@@ -984,6 +1047,10 @@ function load_muplugin_textdomain( $domain, $mu_plugin_rel_path = '' ) {
 	/** @var WP_Textdomain_Registry $wp_textdomain_registry */
 	global $wp_textdomain_registry;
 
+	if ( ! is_string( $domain ) ) {
+		return false;
+	}
+
 	/** This filter is documented in wp-includes/l10n.php */
 	$locale = apply_filters( 'plugin_locale', determine_locale(), $domain );
 
@@ -1023,6 +1090,10 @@ function load_theme_textdomain( $domain, $path = false ) {
 	/** @var WP_Textdomain_Registry $wp_textdomain_registry */
 	global $wp_textdomain_registry;
 
+	if ( ! is_string( $domain ) ) {
+		return false;
+	}
+
 	/**
 	 * Filters a theme's locale.
 	 *
@@ -1136,7 +1207,7 @@ function load_script_textdomain( $handle, $domain = 'default', $path = '' ) {
 		$relative = trim( $relative, '/' );
 		$relative = explode( '/', $relative );
 
-		$languages_path = WP_LANG_DIR . '/' . $relative[0];
+		$languages_path = WP_LANG_DIR . '/plugins';
 
 		$relative = array_slice( $relative, 2 ); // Remove plugins/<plugin name> or themes/<theme name>.
 		$relative = implode( '/', $relative );
@@ -1382,25 +1453,34 @@ function translate_user_role( $name, $domain = 'default' ) {
 }
 
 /**
- * Gets all available languages based on the presence of *.mo files in a given directory.
+ * Gets all available languages based on the presence of *.mo and *.l10n.php files in a given directory.
  *
  * The default directory is WP_LANG_DIR.
  *
  * @since 3.0.0
  * @since 4.7.0 The results are now filterable with the {@see 'get_available_languages'} filter.
+ * @since 6.5.0 The initial file list is now cached and also takes into account *.l10n.php files.
+ *
+ * @global WP_Textdomain_Registry $wp_textdomain_registry WordPress Textdomain Registry.
  *
  * @param string $dir A directory to search for language files.
  *                    Default WP_LANG_DIR.
  * @return string[] An array of language codes or an empty array if no languages are present.
- *                  Language codes are formed by stripping the .mo extension from the language file names.
+ *                  Language codes are formed by stripping the file extension from the language file names.
  */
 function get_available_languages( $dir = null ) {
+	global $wp_textdomain_registry;
+
 	$languages = array();
 
-	$lang_files = glob( ( is_null( $dir ) ? WP_LANG_DIR : $dir ) . '/*.mo' );
+	$path       = is_null( $dir ) ? WP_LANG_DIR : $dir;
+	$lang_files = $wp_textdomain_registry->get_language_files_from_path( $path );
+
 	if ( $lang_files ) {
 		foreach ( $lang_files as $lang_file ) {
 			$lang_file = basename( $lang_file, '.mo' );
+			$lang_file = basename( $lang_file, '.l10n.php' );
+
 			if ( ! str_starts_with( $lang_file, 'continents-cities' ) && ! str_starts_with( $lang_file, 'ms-' ) &&
 				! str_starts_with( $lang_file, 'admin-' ) ) {
 				$languages[] = $lang_file;
@@ -1416,7 +1496,7 @@ function get_available_languages( $dir = null ) {
 	 * @param string[] $languages An array of available language codes.
 	 * @param string   $dir       The directory where the language files were found.
 	 */
-	return apply_filters( 'get_available_languages', $languages, $dir );
+	return apply_filters( 'get_available_languages', array_unique( $languages ), $dir );
 }
 
 /**
@@ -1427,25 +1507,25 @@ function get_available_languages( $dir = null ) {
  *
  * @since 3.7.0
  *
+ * @global WP_Textdomain_Registry $wp_textdomain_registry WordPress Textdomain Registry.
+ *
  * @param string $type What to search for. Accepts 'plugins', 'themes', 'core'.
  * @return array Array of language data.
  */
 function wp_get_installed_translations( $type ) {
+	global $wp_textdomain_registry;
+
 	if ( 'themes' !== $type && 'plugins' !== $type && 'core' !== $type ) {
 		return array();
 	}
 
-	$dir = 'core' === $type ? '' : "/$type";
+	$dir = 'core' === $type ? WP_LANG_DIR : WP_LANG_DIR . "/$type";
 
-	if ( ! is_dir( WP_LANG_DIR ) ) {
+	if ( ! is_dir( $dir ) ) {
 		return array();
 	}
 
-	if ( $dir && ! is_dir( WP_LANG_DIR . $dir ) ) {
-		return array();
-	}
-
-	$files = scandir( WP_LANG_DIR . $dir );
+	$files = $wp_textdomain_registry->get_language_files_from_path( $dir );
 	if ( ! $files ) {
 		return array();
 	}
@@ -1453,16 +1533,7 @@ function wp_get_installed_translations( $type ) {
 	$language_data = array();
 
 	foreach ( $files as $file ) {
-		if ( '.' === $file[0] || is_dir( WP_LANG_DIR . "$dir/$file" ) ) {
-			continue;
-		}
-		if ( ! str_ends_with( $file, '.po' ) ) {
-			continue;
-		}
-		if ( ! preg_match( '/(?:(.+)-)?([a-z]{2,3}(?:_[A-Z]{2})?(?:_[a-z0-9]+)?).po/', $file, $match ) ) {
-			continue;
-		}
-		if ( ! in_array( substr( $file, 0, -3 ) . '.mo', $files, true ) ) {
+		if ( ! preg_match( '/(?:(.+)-)?([a-z]{2,3}(?:_[A-Z]{2})?(?:_[a-z0-9]+)?)\.(?:mo|l10n\.php)/', basename( $file ), $match ) ) {
 			continue;
 		}
 
@@ -1470,7 +1541,25 @@ function wp_get_installed_translations( $type ) {
 		if ( '' === $textdomain ) {
 			$textdomain = 'default';
 		}
-		$language_data[ $textdomain ][ $language ] = wp_get_pomo_file_data( WP_LANG_DIR . "$dir/$file" );
+
+		if ( str_ends_with( $file, '.mo' ) ) {
+			$pofile = substr_replace( $file, '.po', - strlen( '.mo' ) );
+
+			if ( ! file_exists( $pofile ) ) {
+				continue;
+			}
+
+			$language_data[ $textdomain ][ $language ] = wp_get_pomo_file_data( $pofile );
+		} else {
+			$pofile = substr_replace( $file, '.po', - strlen( '.l10n.php' ) );
+
+			// If both a PO and a PHP file exist, prefer the PO file.
+			if ( file_exists( $pofile ) ) {
+				continue;
+			}
+
+			$language_data[ $textdomain ][ $language ] = wp_get_l10n_php_file_data( $file );
+		}
 	}
 	return $language_data;
 }
@@ -1500,6 +1589,41 @@ function wp_get_pomo_file_data( $po_file ) {
 	return $headers;
 }
 
+/**
+ * Extracts headers from a PHP translation file.
+ *
+ * @since 6.6.0
+ *
+ * @param string $php_file Path to a `.l10n.php` file.
+ * @return string[] Array of file header values keyed by header name.
+ */
+function wp_get_l10n_php_file_data( $php_file ) {
+	$data = (array) include $php_file;
+
+	unset( $data['messages'] );
+	$headers = array(
+		'POT-Creation-Date'  => 'pot-creation-date',
+		'PO-Revision-Date'   => 'po-revision-date',
+		'Project-Id-Version' => 'project-id-version',
+		'X-Generator'        => 'x-generator',
+	);
+
+	$result = array(
+		'POT-Creation-Date'  => '',
+		'PO-Revision-Date'   => '',
+		'Project-Id-Version' => '',
+		'X-Generator'        => '',
+	);
+
+	foreach ( $headers as $po_header => $php_header ) {
+		if ( isset( $data[ $php_header ] ) ) {
+			$result[ $po_header ] = $data[ $php_header ];
+		}
+	}
+
+	return $result;
+}
+
 /**
  * Displays or returns a Language selector.
  *
